import functools
import socket
import threading


from yamcs.core.exceptions import YamcsError
from yamcs.core.futures import WebSocketSubscriptionFuture
from yamcs.core.helpers import adapt_name_for_rest, to_isostring
from yamcs.core.subscriptions import WebSocketSubscriptionManager
from yamcs.protobuf import yamcs_pb2
from yamcs.protobuf.pvalue import pvalue_pb2
from yamcs.protobuf.rest import rest_pb2
from yamcs.protobuf.web import web_pb2
from yamcs.protobuf.mdb import mdb_pb2

from yamcs.tmtc.model import (Alarm, AlarmEvent, Calibrator, CommandHistory,
                              IssuedCommand, ParameterData, ParameterValue,
                              RangeSet)


class SequenceGenerator(object):
    """Static atomic counter."""
    _counter = 0
    _lock = threading.Lock()

    @classmethod
    def next(cls):
        with cls._lock:
            cls._counter += 1
            return cls._counter


def _wrap_callback_parse_parameter_data(subscription, on_data, message):
    """
    Wraps an (optional) user callback to parse ParameterData
    from a WebSocket data message
    """
    if message.type == message.REPLY:
        data = web_pb2.ParameterSubscriptionResponse()
        data.ParseFromString(message.reply.data)
        subscription.subscription_id = data.subscriptionId
    elif (message.type == message.DATA and
          message.data.type == yamcs_pb2.PARAMETER):
        parameter_data = ParameterData(getattr(message.data, 'parameterData'))
        #pylint: disable=protected-access
        subscription._process(parameter_data)
        if on_data:
            on_data(parameter_data)


def _wrap_callback_parse_cmdhist_data(subscription, on_data, message):
    """
    Wraps an (optional) user callback to parse CommandHistoryEntry
    from a WebSocket data message
    """
    if (message.type == message.DATA and
            message.data.type == yamcs_pb2.CMD_HISTORY):
        entry = getattr(message.data, 'command')
        #pylint: disable=protected-access
        rec = subscription._process(entry)
        if on_data:
            on_data(rec)


def _wrap_callback_parse_alarm_data(subscription, on_data, message):
    """
    Wraps an (optional) user callback to parse Alarm data
    from a WebSocket data message
    """
    if (message.type == message.DATA and
            message.data.type == yamcs_pb2.ALARM_DATA):
        proto = getattr(message.data, 'alarmData')
        alarm_event = AlarmEvent(proto)
        #pylint: disable=protected-access
        subscription._process(alarm_event)
        if on_data:
            on_data(alarm_event)


def _build_named_object_id(parameter):
    """
    Builds a NamedObjectId. This is a bit more complex than it really
    should be. In Python (for convenience) we allow the user to simply address
    entries by their alias via the NAMESPACE/NAME convention. Yamcs is not
    aware of this convention so we decompose it into distinct namespace and
    name fields.
    """
    named_object_id = yamcs_pb2.NamedObjectId()
    if parameter.startswith('/'):
        named_object_id.name = parameter
    else:
        parts = parameter.split('/', 1)
        if len(parts) < 2:
            raise ValueError('Failed to process {}. Use fully-qualified '
                             'XTCE names or, alternatively, an alias in '
                             'in the format NAMESPACE/NAME'.format(parameter))
        named_object_id.namespace = parts[0]
        named_object_id.name = parts[1]
    return named_object_id


def _build_named_object_ids(parameters):
    """Builds a list of NamedObjectId."""
    if isinstance(parameters, str):
        return [_build_named_object_id(parameters)]
    return [_build_named_object_id(parameter) for parameter in parameters]


#pylint: disable=protected-access
def _build_command_ids(issued_commands):
    """Builds a list of CommandId."""
    if isinstance(issued_commands, IssuedCommand):
        entry = issued_commands._proto.commandQueueEntry
        return [entry.cmdId]
    else:
        return [issued_command._proto.commandQueueEntry.cmdId
                for issued_command in issued_commands]


def _build_value_proto(value):
    proto = yamcs_pb2.Value()
    if isinstance(value, bool):
        proto.type = proto.BOOLEAN
        proto.booleanValue = value
    elif isinstance(value, float):
        proto.type = proto.FLOAT
        proto.floatValue = value
    elif isinstance(value, int) and value > 2147483647:
        proto.type = proto.SINT64
        proto.sint64Value = value
    elif isinstance(value, int):
        proto.type = proto.SINT32
        proto.sint32Value = value
    elif isinstance(value, str):
        proto.type = proto.STRING
        proto.stringValue = value
    else:
        raise YamcsError('Unrecognized type')
    return proto

def _set_range(ar, range, level):
    ar.level = level
    if range[0] :
        ar.minExclusive = range[0]
    if range[1] :
        ar.maxExclusive = range[1]
    
def _add_alarms(alarm_info, watch, warning, distress, critical, severe, min_violations):
    alarm_info.minViolations = min_violations
    
    if watch:
        ar = alarm_info.staticAlarmRange.add()
        _set_range(ar, watch, mdb_pb2.WATCH)
    if warning:
        ar = alarm_info.staticAlarmRange.add()
        _set_range(ar, warning, mdb_pb2.WARNING)    
    if distress:
        ar = alarm_info.staticAlarmRange.add()
        _set_range(ar, distress, mdb_pb2.DISTRESS)
    
    if critical:
        ar = alarm_info.staticAlarmRange.add()
        _set_range(ar, critical, mdb_pb2.CRITICAL)
    if severe:
        ar = alarm_info.staticAlarmRange.add()
        _set_range(ar, severe, mdb_pb2.SEVERE)
        
def _add_calib(calib_info, type, data):
    type = type.lower()
    if type == Calibrator.POLYNOMIAL:           
        calib_info.type = mdb_pb2.CalibratorInfo.POLYNOMIAL
        polynomial = calib_info.polynomialCalibrator.coefficient.extend(data)           
    elif type == Calibrator.SPLINE:
        calib_info.type = CalibratorInfo.Type.SPLINE
        spline = mdb_pb2.SplineCalibratorInfo()
        for p in data:
            spi = spline.point.add()
            spi.raw = p[0]
            spi.calibrated = p[1]
    else :
        raise YamcsError('Unrecognized type')


class CommandHistorySubscription(WebSocketSubscriptionFuture):
    """
    Local object providing access to command history updates.

    This object buffers all received command history. This is needed
    to stitch together incremental command history events.

    If you expect to receive a lot of command history updates
    you should periodically clear local cache via ``clear_cache()``.
    In future work, we may add automated buffer management within
    configurable watermarks.

    .. warning::
        If command history updates are received for commands
        that are not currently in the local cache, the returned
        information may be incomplete.
    """

    @staticmethod
    def _cache_key(cmd_id):
        """commandId is a tuple. Make a 'unique' key for it."""
        return '{}__{}__{}__{}'.format(
            cmd_id.generationTime, cmd_id.origin, cmd_id.sequenceNumber,
            cmd_id.commandName)

    def __init__(self, manager):
        super(CommandHistorySubscription, self).__init__(manager)
        self._cache = {}

    def clear_cache(self):
        """
        Clears local command history cache.
        """
        self._cache = {}

    def get_command_history(self, issued_command):
        """
        Gets locally cached CommandHistory for the specified command.

        :param .IssuedCommand issued_command: object representing a
                                              previously issued command.
        :rtype: .CommandHistory
        """
        #pylint: disable=protected-access
        entry = issued_command._proto.commandQueueEntry
        key = self._cache_key(entry.cmdId)
        if key in self._cache:
            return self._cache[key]
        return None

    def _process(self, entry):
        key = self._cache_key(entry.commandId)
        if key in self._cache:
            cmdhist = self._cache[key]
            #pylint: disable=protected-access
            cmdhist._update(entry.attr)
        else:
            cmdhist = CommandHistory(entry)
            self._cache[key] = cmdhist

        return cmdhist


class ParameterSubscription(WebSocketSubscriptionFuture):
    """
    Local object representing a subscription of zero or more parameters.

    A subscription object stores the last received value of each
    subscribed parameter.
    """

    def __init__(self, manager):
        super(ParameterSubscription, self).__init__(manager)

        self.value_cache = {}
        """Value cache keyed by parameter name."""

        self.delivery_count = 0
        """The number of parameter deliveries."""

        # The actual subscription_id is set async after server reply
        self.subscription_id = -1
        """Subscription number assigned by the server. This is set async,
        so may not be immediately available."""

    def add(self,
            parameters,
            abort_on_invalid=True,
            send_from_cache=True):
        """
        Add one or more parameters to this subscription.

        :param parameters: Parameter(s) to be added
        :type parameters: Union[str, str[]]
        :param bool abort_on_invalid: If ``True`` one invalid parameter
                                      means any other parameter in the
                                      request will also not be added
                                      to the subscription.
        :param bool send_from_cache: If ``True`` the last processed parameter
                                     value is sent from parameter cache.
                                     When ``False`` only newly processed
                                     parameters are received.
        """

        # Verify that we already know our assigned subscription_id
        assert self.subscription_id != -1

        if not parameters:
            return

        options = web_pb2.ParameterSubscriptionRequest()
        options.subscriptionId = self.subscription_id
        options.abortOnInvalid = abort_on_invalid
        options.sendFromCache = send_from_cache
        options.id.extend(_build_named_object_ids(parameters))

        self._manager.send('subscribe', options)

    def remove(self, parameters):
        """
        Remove one or more parameters from this subscription.

        :param parameters: Parameter(s) to be removed
        :type parameters: Union[str, str[]]
        """

        # Verify that we already know our assigned subscription_id
        assert self.subscription_id != -1

        if not parameters:
            return

        options = web_pb2.ParameterSubscriptionRequest()
        options.subscriptionId = self.subscription_id
        options.id.extend(_build_named_object_ids(parameters))

        self._manager.send('unsubscribe', options)

    def get_value(self, parameter):
        """
        Returns the last value of a specific parameter from local cache.

        :rtype: .ParameterValue
        """
        return self.value_cache[parameter]

    def _process(self, parameter_data):
        self.delivery_count += 1
        for pval in parameter_data.parameters:
            self.value_cache[pval.name] = pval


class AlarmSubscription(WebSocketSubscriptionFuture):
    """
    Local object representing an alarm subscription.

    A subscription object stores the currently active
    alarms. There can be only one alarm active at a
    time per parameter.
    """

    def __init__(self, manager):
        super(AlarmSubscription, self).__init__(manager)

        self._cache = {}
        """Value cache keyed by parameter name."""

    def get_alarm(self, parameter):
        """
        Returns the alarm state associated with a specific parameter from
        local cache.

        :param str parameter: Fully-qualified XTCE name
        :rtype: .Alarm
        """
        return self._cache[parameter]

    def list_alarms(self):
        """
        Returns a snapshot of all active alarms.

        :param str parameter: Fully-qualified XTCE name
        :rtype: .Alarm[]
        """
        return [self._cache[k] for k in self._cache]

    def _process(self, alarm_event):
        alarm = alarm_event.alarm
        if alarm_event.event_type in ('ACKNOWLEDGED', 'CLEARED'):
            del self._cache[alarm.name]
        else:
            self._cache[alarm.name] = alarm


class ProcessorClient(object):
    """Client object that groups operations linked to a specific processor."""

    def __init__(self, client, instance, processor):
        super(ProcessorClient, self).__init__()
        self._client = client
        self._instance = instance
        self._processor = processor

    def get_parameter_value(self, parameter, from_cache=True, timeout=10):
        """
        Retrieve the current value of the specified parameter.

        :param str parameter: Either a fully-qualified XTCE name or an alias in the
                              format ``NAMESPACE/NAME``.
        :param bool from_cache: If ``False`` this call will block until a
                                fresh value is received on the processor.
                                If ``True`` the server returns the latest
                                value instead (which may be ``None``).
        :param float timeout: The amount of seconds to wait for a fresh value.
                              (ignored if ``from_cache=True``).
        :rtype: .ParameterValue
        """
        params = {
            'fromCache': from_cache,
            'timeout': int(timeout * 1000),
        }
        parameter = adapt_name_for_rest(parameter)
        url = '/processors/{}/{}/parameters{}'.format(
            self._instance, self._processor, parameter)
        response = self._client.get_proto(url, params=params)
        proto = pvalue_pb2.ParameterValue()
        proto.ParseFromString(response.content)

        # Server returns ParameterValue with only 'id' set if no
        # value existed. Convert this to ``None``.
        if proto.HasField('rawValue') or proto.HasField('engValue'):
            return ParameterValue(proto)
        return None

    def get_parameter_values(self, parameters, from_cache=True, timeout=10):
        """
        Retrieve the current value of the specified parameter.

        :param str[] parameters: List of parameter names. These may be
                                 fully-qualified XTCE name or an alias
                                 in the format ``NAMESPACE/NAME``.
        :param bool from_cache: If ``False`` this call will block until
                                fresh values are received on the processor.
                                If ``True`` the server returns the latest
                                value instead (which may be ``None``).
        :param float timeout: The amount of seconds to wait for a fresh
                              values (ignored if ``from_cache=True``).
        :return: A list that matches the length and order of the requested
                 list of parameters. Each entry contains either the
                 returned parameter value, or ``None``.
        :rtype: .ParameterValue[]
        """
        params = {
            'fromCache': from_cache,
            'timeout': int(timeout * 1000),
        }
        req = rest_pb2.BulkGetParameterValueRequest()
        req.id.extend(_build_named_object_ids(parameters))
        url = '/processors/{}/{}/parameters/mget'.format(
            self._instance, self._processor)
        response = self._client.post_proto(url, params=params,
                                           data=req.SerializeToString())
        proto = rest_pb2.BulkGetParameterValueResponse()
        proto.ParseFromString(response.content)

        pvals = []
        for parameter_id in req.id:
            match = None
            for pval in proto.value:
                if pval.id == parameter_id:
                    match = pval
                    break
            pvals.append(ParameterValue(match) if match else None)
        return pvals

    def set_parameter_value(self, parameter, value):
        """
        Sets the value of the specified parameter.

        :param str parameter: Either a fully-qualified XTCE name or an alias in the
                              format ``NAMESPACE/NAME``.
        :param value: The value to set
        """
        parameter = adapt_name_for_rest(parameter)
        url = '/processors/{}/{}/parameters{}'.format(
            self._instance, self._processor, parameter)
        req = _build_value_proto(value)
        self._client.put_proto(url, data=req.SerializeToString())

    def set_parameter_values(self, values):
        """
        Sets the value of multiple  parameters.

        :param dict values: Values keyed by parameter name. This name can be either
                            a fully-qualified XTCE name or an alias in the format
                            ``NAMESPACE/NAME``.
        """
        req = rest_pb2.BulkSetParameterValueRequest()
        for key in values:
            item = req.request.add()
            item.id.MergeFrom(_build_named_object_id(key))
            item.value.MergeFrom(_build_value_proto(values[key]))
        url = '/processors/{}/{}/parameters/mset'.format(
            self._instance, self._processor)
        self._client.post_proto(url, data=req.SerializeToString())

    def issue_command(self, command, args=None, dry_run=False, comment=None):
        """
        Issue the given command

        :param str command: Either a fully-qualified XTCE name or an alias in the
                            format ``NAMESPACE/NAME``.
        :param dict args: named arguments (if the command requires these)
        :param bool dry_run: If ``True`` the command is not actually issued. This
                             can be used to check if the server would generate
                             errors when preparing the command (for example
                             because an argument is missing).
        :param str comment: Comment attached to the command.
        :return: An object providing access to properties of the newly issued
                 command.
        :rtype: .IssuedCommand
        """
        req = rest_pb2.IssueCommandRequest()
        req.sequenceNumber = SequenceGenerator.next()
        req.origin = socket.gethostname()
        req.dryRun = dry_run
        if comment:
            req.comment = comment
        if args:
            for key in args:
                assignment = req.assignment.add()
                assignment.name = key
                assignment.value = str(args[key])

        command = adapt_name_for_rest(command)
        url = '/processors/{}/{}/commands{}'.format(
            self._instance, self._processor, command)
        response = self._client.post_proto(url, data=req.SerializeToString())
        proto = rest_pb2.IssueCommandResponse()
        proto.ParseFromString(response.content)
        return IssuedCommand(proto, self)

    def list_alarms(self, start=None, stop=None):
        """
        Lists the active alarms.

        Remark that this does not query the archive. Only active alarms on the
        current processor are returned.

        :param ~datetime.datetime start: Minimum trigger time of the returned alarms (inclusive)
        :param ~datetime.datetime stop: Maximum trigger time of the returned alarms (exclusive)
        :rtype: ~collections.Iterable[.Alarm]
        """
        # TODO implement continuation token on server
        params = {
            'order': 'asc'
        }
        if start is not None:
            params['start'] = to_isostring(start)
        if stop is not None:
            params['stop'] = to_isostring(stop)
        # Server does not do pagination on listings of this resource.
        # Return an iterator anyway for similarity with other API methods
        url = '/processors/{}/{}/alarms'.format(self._instance, self._processor)
        response = self._client.get_proto(path=url, params=params)
        message = rest_pb2.ListAlarmsResponse()
        message.ParseFromString(response.content)
        alarms = getattr(message, 'alarm')
        return iter([Alarm(alarm) for alarm in alarms])

    def set_default_calibrator(self, parameter, type, data):  # pylint: disable=W0622
        """
        Apply a calibrator while processing raw values of the specified
        parameter. If there is already a default calibrator associated
        to this parameter, that calibrator gets replaced.

        .. note::

            Contextual calibrators take precedence over the default calibrator
            See :meth:`set_calibrators` for setting contextual calibrators.

        Two types of calibrators can be applied:

        * Polynomial calibrators apply a polynomial expression of the form:
          `y = a + bx + cx^2 + ...`.

          The `data` argument must be an array of floats ``[a, b, c, ...]``.

        * Spline calibrators interpolate the raw value between a set of points
          which represent a linear curve.

          The `data` argument must be an array of ``[x, y]`` points.

        :param str parameter: Either a fully-qualified XTCE name or an alias
                              in the format ``NAMESPACE/NAME``.
        :param str type: One of ``polynomial`` or ``spline``.
        :param data: Calibration definition for the selected type.
        """
        req = mdb_pb2.ChangeParameterRequest()
        req.action = mdb_pb2.ChangeParameterRequest.SET_DEFAULT_CALIBRATOR
        if  type:
            _add_calib(req.defaultCalibrator, type, data)
                   
        url = '/mdb/{}/{}/parameters/{}'.format(
            self._instance, self._processor, parameter)
        response = self._client.post_proto(url, data=req.SerializeToString())
       # pti = mdb_pb2.ParameterTypeInfo()
       # pti.ParseFromString(response.content)
       # print(pti)
       

     

    def set_calibrators(self, parameter, calibrators):
        """
        Apply an ordered set of calibrators for the specified parameter.
        This replaces existing calibrators (if any).

        Each calibrator may have a context, which indicates when it its
        effects may be applied. Only the first matching calibrator is
        applied.

        A calibrator with context ``None`` is the *default* calibrator.
        There can be only one such calibrator, and is always applied at
        the end when no other contextual calibrator was applicable.

        :param str parameter: Either a fully-qualified XTCE name or an alias
                              in the format ``NAMESPACE/NAME``.
        :param .Calibrator[] calibrators: List of calibrators (either contextual or not)
        """
        req = mdb_pb2.ChangeParameterRequest()
        req.action = mdb_pb2.ChangeParameterRequest.SET_CALIBRATORS
        for c in calibrators:
            if c.context :
                context_calib = req.contextCalibrator.add()
                context_calib.context = rs.context
                calib_info = context_calib.calibrator
            else :                
                calib_info = req.defaultCalibrator
            
            _add_calib(calib_info, c.type, c.data)
            
            
        url = '/mdb/{}/{}/parameters/{}'.format(
            self._instance, self._processor, parameter)
        response = self._client.post_proto(url, data=req.SerializeToString())
        pti = mdb_pb2.ParameterTypeInfo()
      #  pti.ParseFromString(response.content)
      # print(pti)

    def clear_calibrators(self, parameter):
        """
        Removes all calibrators for the specified parameter.
        """
        self.set_default_calibrator(parameter, None, None)
        self.set_calibrators(parameter, [])
        
        
    def reset_calibrators(self, parameter):
        """
        Reset all calibrators for the specified parameter to their original MDB value.
        """
        req = mdb_pb2.ChangeParameterRequest()
        req.action = mdb_pb2.ChangeParameterRequest.RESET_CALIBRATORS                  
        calib_info = req.defaultCalibrator
        url = '/mdb/{}/{}/parameters/{}'.format(
            self._instance, self._processor, parameter)
        response = self._client.post_proto(url, data=req.SerializeToString())
        

    def set_default_alarm_ranges(self, parameter, watch=None, warning=None,
                                 distress=None, critical=None, severe=None,
                                 min_violations=1):
        """
        Generate out-of-limit alarms for a parameter using the specified
        alarm ranges.

        This replaces any previous default alarms on this parameter.

        .. note::

            Contextual range sets take precedence over the default alarm
            ranges. See :meth:`set_alarm_range_sets` for setting contextual
            range sets.

        :param str parameter: Either a fully-qualified XTCE name or an alias
                              in the format ``NAMESPACE/NAME``.
        :param (float,float) watch: Range expressed as a tuple ``(lo, hi)``
                                where lo and hi are assumed exclusive.
        :param (float,float) warning: Range expressed as a tuple ``(lo, hi)``
                                  where lo and hi are assumed exclusive.
        :param (float,float) distress: Range expressed as a tuple ``(lo, hi)``
                                   where lo and hi are assumed exclusive.
        :param (float,float) critical: Range expressed as a tuple ``(lo, hi)``
                                   where lo and hi are assumed exclusive.
        :param (float,float) severe: Range expressed as a tuple ``(lo, hi)``
                                 where lo and hi are assumed exclusive.
        :param int min_violations: Minimum violations before an alarm is
                                   generated.
        """
        req = mdb_pb2.ChangeParameterRequest()
        req.action = mdb_pb2.ChangeParameterRequest.SET_DEFAULT_ALARMS
        if(watch or warning or distress or critical or severe):
            _add_alarms(req.defaultAlarm, watch, warning, distress, critical, severe, min_violations)
        
        url = '/mdb/{}/{}/parameters/{}'.format(
            self._instance, self._processor, parameter)
        response = self._client.post_proto(url, data=req.SerializeToString())
       # pti = mdb_pb2.ParameterTypeInfo()
       # pti.ParseFromString(response.content)
       # print(pti)
        

    def set_alarm_range_sets(self, parameter, sets):
        """
        Apply an ordered list of alarm range sets for the specified parameter.
        This replaces existing alarm sets (if any).

        Each RangeSet may have a context, which indicates when
        its effects may be applied. Only the first matching set is
        applied.

        A RangeSet with context ``None`` represents the *default* set of
        alarm ranges.  There can be only one such set, and it is always
        applied at the end when no other set of contextual ranges is
        applicable.

        :param str parameter: Either a fully-qualified XTCE name or an alias
                              in the format ``NAMESPACE/NAME``.
        :param .RangeSet[] sets: List of range sets (either contextual or not)
        """
        req = mdb_pb2.ChangeParameterRequest()
        req.action = mdb_pb2.ChangeParameterRequest.SET_ALARMS
        for rs in sets:
            if rs.context :
                context_alarm = req.contextAlarm.add()
                context_alarm.context = rs.context
                alarm_info = context_alarm.alarm
            else :                
                alarm_info = req.defaultAlarm
                
            _add_alarms(alarm_info, rs.watch, rs.warning, rs.distress, rs.critical, rs.severe, rs.min_violations)
            
        url = '/mdb/{}/{}/parameters/{}'.format(
            self._instance, self._processor, parameter)
        response = self._client.post_proto(url, data=req.SerializeToString())
        pti = mdb_pb2.ParameterTypeInfo()
        pti.ParseFromString(response.content)
        print(pti)

    def clear_alarm_ranges(self, parameter):
        """
        Removes all alarm limits for the specified parameter.
        """
        self.set_default_alarm_ranges(parameter)
        self.set_alarm_range_sets(parameter, [])
        
        
    def reset_alarm_ranges(self, parameter):
        """
        Reset all alarm limits for the specified parameter to their original MDB value.
        """
        req = mdb_pb2.ChangeParameterRequest()
        req.action = mdb_pb2.ChangeParameterRequest.RESET_ALARMS        
        
        url = '/mdb/{}/{}/parameters/{}'.format(
            self._instance, self._processor, parameter)
        response = self._client.post_proto(url, data=req.SerializeToString())
        #pti = mdb_pb2.ParameterTypeInfo()
        #pti.ParseFromString(response.content)
        #print(pti)
       
        

    def acknowledge_alarm(self, alarm, comment=None):
        """
        Acknowledges a specific alarm associated with a parameter.

        :param alarm: Alarm instance
        :type alarm: :class:`.Alarm`
        :param str comment: Optional comment to associate with the state
                            change.
        """
        url = '/processors/{}/{}/parameters{}/alarms/{}'.format(
            self._instance, self._processor, alarm.name, alarm.sequence_number)
        req = rest_pb2.EditAlarmRequest()
        req.state = 'acknowledged'
        if comment is not None:
            req.comment = comment
        self._client.put_proto(url, data=req.SerializeToString())

    def create_command_history_subscription(self,
                                            issued_command=None,
                                            on_data=None,
                                            timeout=60):
        """
        Create a new command history subscription.

        :param .IssuedCommand[] issued_command: (Optional) Previously issued
                                                commands. If not provided updates
                                                from any command are received.
        :param on_data: Function that gets called with  :class:`.CommandHistory`
                        updates.
        :param float timeout: The amount of seconds to wait for the request
                              to complete.
        :return: Future that can be used to manage the background websocket
                 subscription
        :rtype: .CommandHistorySubscription
        """
        options = web_pb2.CommandHistorySubscriptionRequest()
        options.ignorePastCommands = True
        if issued_command:
            options.commandId.extend(_build_command_ids(issued_command))

        manager = WebSocketSubscriptionManager(
            self._client, resource='cmdhistory', options=options)

        # Represent subscription as a future
        subscription = CommandHistorySubscription(manager)

        wrapped_callback = functools.partial(
            _wrap_callback_parse_cmdhist_data, subscription, on_data)

        manager.open(wrapped_callback, instance=self._instance,
                     processor=self._processor)

        # Wait until a reply or exception is received
        subscription.reply(timeout=timeout)

        return subscription

    def create_parameter_subscription(self,
                                      parameters,
                                      on_data=None,
                                      abort_on_invalid=True,
                                      update_on_expiration=False,
                                      send_from_cache=True,
                                      timeout=60):
        """
        Create a new parameter subscription.

        :param str[] parameters: Parameter names (or aliases).
        :param on_data: Function that gets called with  :class:`.ParameterData`
                        updates.
        :param bool abort_on_invalid: If ``True`` an error is generated when
                                      invalid parameters are specified.
        :param bool update_on_expiration: If ``True`` an update is received
                                          when a parameter value has become
                                          expired. This update holds the
                                          same value as the last known valid
                                          value, but with status set to
                                          ``EXPIRED``.
        :param bool send_from_cache: If ``True`` the last processed parameter
                                     value is sent from parameter cache.
                                     When ``False`` only newly processed
                                     parameters are received.
        :param float timeout: The amount of seconds to wait for the request
                              to complete.

        :return: A Future that can be used to manage the background websocket
                 subscription.
        :rtype: .ParameterSubscription
        """
        options = web_pb2.ParameterSubscriptionRequest()
        options.subscriptionId = -1  # This means 'create a new subscription'
        options.abortOnInvalid = abort_on_invalid
        options.updateOnExpiration = update_on_expiration
        options.sendFromCache = send_from_cache
        options.id.extend(_build_named_object_ids(parameters))

        manager = WebSocketSubscriptionManager(
            self._client, resource='parameter', options=options)

        # Represent subscription as a future
        subscription = ParameterSubscription(manager)

        wrapped_callback = functools.partial(
            _wrap_callback_parse_parameter_data, subscription, on_data)

        manager.open(wrapped_callback, instance=self._instance,
                     processor=self._processor)

        # Wait until a reply or exception is received
        subscription.reply(timeout=timeout)

        return subscription

    def create_alarm_subscription(self,
                                  on_data=None,
                                  timeout=60):
        """
        Create a new alarm subscription.

        :param on_data: Function that gets called with  :class:`.AlarmEvent`
                        updates.
        :param float timeout: The amount of seconds to wait for the request
                              to complete.

        :return: A Future that can be used to manage the background websocket
                 subscription.
        :rtype: .AlarmSubscription
        """
        manager = WebSocketSubscriptionManager(self._client, resource='alarms')

        # Represent subscription as a future
        subscription = AlarmSubscription(manager)

        wrapped_callback = functools.partial(
            _wrap_callback_parse_alarm_data, subscription, on_data)

        manager.open(wrapped_callback, instance=self._instance,
                     processor=self._processor)

        # Wait until a reply or exception is received
        subscription.reply(timeout=timeout)

        return subscription
    
    def set_algorithm(self, parameter, text):
        """
        Change an algorithm text. Can only be peformed on JavaScript or Python algorithms.
        
        :param string text: new algorithm text (as it would appear in excel or XTCE)
        
        :param str parameter: Either a fully-qualified XTCE name or an alias
                              in the format ``NAMESPACE/NAME``.
        """
        req = mdb_pb2.ChangeAlgorithmRequest()
        
        req.action = mdb_pb2.ChangeAlgorithmRequest.SET
        req.algorithm.text = text
        
        url = '/mdb/{}/{}/algorithms/{}'.format(
            self._instance, self._processor, parameter)
        response = self._client.post_proto(url, data=req.SerializeToString())
        
    def reset_algorithm(self, parameter):
        """
        Reset the algorithm text to its original definition from MDB
        
        :param str parameter: Either a fully-qualified XTCE name or an alias
                              in the format ``NAMESPACE/NAME``.
        """
        req = mdb_pb2.ChangeAlgorithmRequest()
        
        req.action = mdb_pb2.ChangeAlgorithmRequest.RESET        
        
        url = '/mdb/{}/{}/algorithms/{}'.format(
            self._instance, self._processor, parameter)
        response = self._client.post_proto(url, data=req.SerializeToString())
        



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: yamcs/protobuf/alarms/alarms.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from yamcs.protobuf.mdb import mdb_pb2 as yamcs_dot_protobuf_dot_mdb_dot_mdb__pb2
from yamcs.protobuf.pvalue import pvalue_pb2 as yamcs_dot_protobuf_dot_pvalue_dot_pvalue__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='yamcs/protobuf/alarms/alarms.proto',
  package='yamcs.protobuf.alarms',
  syntax='proto2',
  serialized_options=_b('\n\022org.yamcs.protobuf'),
  serialized_pb=_b('\n\"yamcs/protobuf/alarms/alarms.proto\x12\x15yamcs.protobuf.alarms\x1a\x1cyamcs/protobuf/mdb/mdb.proto\x1a\"yamcs/protobuf/pvalue/pvalue.proto\"z\n\x0f\x41\x63knowledgeInfo\x12\x16\n\x0e\x61\x63knowledgedBy\x18\x01 \x01(\t\x12\x1a\n\x12\x61\x63knowledgeMessage\x18\x02 \x01(\t\x12\x17\n\x0f\x61\x63knowledgeTime\x18\x03 \x01(\x03\x12\x1a\n\x12\x61\x63knowledgeTimeUTC\x18\x04 \x01(\t\"\x81\x04\n\tAlarmData\x12\x0e\n\x06seqNum\x18\x01 \x01(\r\x12\x33\n\x04type\x18\x02 \x01(\x0e\x32%.yamcs.protobuf.alarms.AlarmData.Type\x12;\n\x0ctriggerValue\x18\x03 \x01(\x0b\x32%.yamcs.protobuf.pvalue.ParameterValue\x12>\n\x0fmostSevereValue\x18\x04 \x01(\x0b\x32%.yamcs.protobuf.pvalue.ParameterValue\x12;\n\x0c\x63urrentValue\x18\x05 \x01(\x0b\x32%.yamcs.protobuf.pvalue.ParameterValue\x12\x12\n\nviolations\x18\x06 \x01(\r\x12?\n\x0f\x61\x63knowledgeInfo\x18\x07 \x01(\x0b\x32&.yamcs.protobuf.alarms.AcknowledgeInfo\x12\x34\n\tparameter\x18\x08 \x01(\x0b\x32!.yamcs.protobuf.mdb.ParameterInfo\"j\n\x04Type\x12\n\n\x06\x41\x43TIVE\x10\x01\x12\r\n\tTRIGGERED\x10\x02\x12\x16\n\x12SEVERITY_INCREASED\x10\x03\x12\x10\n\x0cPVAL_UPDATED\x10\x04\x12\x10\n\x0c\x41\x43KNOWLEDGED\x10\x05\x12\x0b\n\x07\x43LEARED\x10\x06\x42\x14\n\x12org.yamcs.protobuf')
  ,
  dependencies=[yamcs_dot_protobuf_dot_mdb_dot_mdb__pb2.DESCRIPTOR,yamcs_dot_protobuf_dot_pvalue_dot_pvalue__pb2.DESCRIPTOR,])



_ALARMDATA_TYPE = _descriptor.EnumDescriptor(
  name='Type',
  full_name='yamcs.protobuf.alarms.AlarmData.Type',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ACTIVE', index=0, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TRIGGERED', index=1, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SEVERITY_INCREASED', index=2, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PVAL_UPDATED', index=3, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ACKNOWLEDGED', index=4, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CLEARED', index=5, number=6,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=659,
  serialized_end=765,
)
_sym_db.RegisterEnumDescriptor(_ALARMDATA_TYPE)


_ACKNOWLEDGEINFO = _descriptor.Descriptor(
  name='AcknowledgeInfo',
  full_name='yamcs.protobuf.alarms.AcknowledgeInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='acknowledgedBy', full_name='yamcs.protobuf.alarms.AcknowledgeInfo.acknowledgedBy', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='acknowledgeMessage', full_name='yamcs.protobuf.alarms.AcknowledgeInfo.acknowledgeMessage', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='acknowledgeTime', full_name='yamcs.protobuf.alarms.AcknowledgeInfo.acknowledgeTime', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='acknowledgeTimeUTC', full_name='yamcs.protobuf.alarms.AcknowledgeInfo.acknowledgeTimeUTC', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=127,
  serialized_end=249,
)


_ALARMDATA = _descriptor.Descriptor(
  name='AlarmData',
  full_name='yamcs.protobuf.alarms.AlarmData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='seqNum', full_name='yamcs.protobuf.alarms.AlarmData.seqNum', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='yamcs.protobuf.alarms.AlarmData.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='triggerValue', full_name='yamcs.protobuf.alarms.AlarmData.triggerValue', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mostSevereValue', full_name='yamcs.protobuf.alarms.AlarmData.mostSevereValue', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='currentValue', full_name='yamcs.protobuf.alarms.AlarmData.currentValue', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='violations', full_name='yamcs.protobuf.alarms.AlarmData.violations', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='acknowledgeInfo', full_name='yamcs.protobuf.alarms.AlarmData.acknowledgeInfo', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='parameter', full_name='yamcs.protobuf.alarms.AlarmData.parameter', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ALARMDATA_TYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=252,
  serialized_end=765,
)

_ALARMDATA.fields_by_name['type'].enum_type = _ALARMDATA_TYPE
_ALARMDATA.fields_by_name['triggerValue'].message_type = yamcs_dot_protobuf_dot_pvalue_dot_pvalue__pb2._PARAMETERVALUE
_ALARMDATA.fields_by_name['mostSevereValue'].message_type = yamcs_dot_protobuf_dot_pvalue_dot_pvalue__pb2._PARAMETERVALUE
_ALARMDATA.fields_by_name['currentValue'].message_type = yamcs_dot_protobuf_dot_pvalue_dot_pvalue__pb2._PARAMETERVALUE
_ALARMDATA.fields_by_name['acknowledgeInfo'].message_type = _ACKNOWLEDGEINFO
_ALARMDATA.fields_by_name['parameter'].message_type = yamcs_dot_protobuf_dot_mdb_dot_mdb__pb2._PARAMETERINFO
_ALARMDATA_TYPE.containing_type = _ALARMDATA
DESCRIPTOR.message_types_by_name['AcknowledgeInfo'] = _ACKNOWLEDGEINFO
DESCRIPTOR.message_types_by_name['AlarmData'] = _ALARMDATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

AcknowledgeInfo = _reflection.GeneratedProtocolMessageType('AcknowledgeInfo', (_message.Message,), dict(
  DESCRIPTOR = _ACKNOWLEDGEINFO,
  __module__ = 'yamcs.protobuf.alarms.alarms_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.alarms.AcknowledgeInfo)
  ))
_sym_db.RegisterMessage(AcknowledgeInfo)

AlarmData = _reflection.GeneratedProtocolMessageType('AlarmData', (_message.Message,), dict(
  DESCRIPTOR = _ALARMDATA,
  __module__ = 'yamcs.protobuf.alarms.alarms_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.alarms.AlarmData)
  ))
_sym_db.RegisterMessage(AlarmData)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: yamcs/protobuf/tse/tse.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from yamcs.protobuf import yamcs_pb2 as yamcs_dot_protobuf_dot_yamcs__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='yamcs/protobuf/tse/tse.proto',
  package='yamcs.protobuf.tse',
  syntax='proto2',
  serialized_options=_b('\n\022org.yamcs.protobuf'),
  serialized_pb=_b('\n\x1cyamcs/protobuf/tse/tse.proto\x12\x12yamcs.protobuf.tse\x1a\x1ayamcs/protobuf/yamcs.proto\"\xe9\x02\n\nTseCommand\x12\x12\n\ninstrument\x18\x01 \x01(\t\x12\x0f\n\x07\x63ommand\x18\x02 \x01(\t\x12\x10\n\x08response\x18\x03 \x01(\t\x12L\n\x0f\x61rgumentMapping\x18\x04 \x03(\x0b\x32\x33.yamcs.protobuf.tse.TseCommand.ArgumentMappingEntry\x12N\n\x10parameterMapping\x18\x05 \x03(\x0b\x32\x34.yamcs.protobuf.tse.TseCommand.ParameterMappingEntry\x1aM\n\x14\x41rgumentMappingEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12$\n\x05value\x18\x02 \x01(\x0b\x32\x15.yamcs.protobuf.Value:\x02\x38\x01\x1a\x37\n\x15ParameterMappingEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\x14\n\x12org.yamcs.protobuf')
  ,
  dependencies=[yamcs_dot_protobuf_dot_yamcs__pb2.DESCRIPTOR,])




_TSECOMMAND_ARGUMENTMAPPINGENTRY = _descriptor.Descriptor(
  name='ArgumentMappingEntry',
  full_name='yamcs.protobuf.tse.TseCommand.ArgumentMappingEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='yamcs.protobuf.tse.TseCommand.ArgumentMappingEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='yamcs.protobuf.tse.TseCommand.ArgumentMappingEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=308,
  serialized_end=385,
)

_TSECOMMAND_PARAMETERMAPPINGENTRY = _descriptor.Descriptor(
  name='ParameterMappingEntry',
  full_name='yamcs.protobuf.tse.TseCommand.ParameterMappingEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='yamcs.protobuf.tse.TseCommand.ParameterMappingEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='yamcs.protobuf.tse.TseCommand.ParameterMappingEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=387,
  serialized_end=442,
)

_TSECOMMAND = _descriptor.Descriptor(
  name='TseCommand',
  full_name='yamcs.protobuf.tse.TseCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instrument', full_name='yamcs.protobuf.tse.TseCommand.instrument', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='command', full_name='yamcs.protobuf.tse.TseCommand.command', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='response', full_name='yamcs.protobuf.tse.TseCommand.response', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='argumentMapping', full_name='yamcs.protobuf.tse.TseCommand.argumentMapping', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='parameterMapping', full_name='yamcs.protobuf.tse.TseCommand.parameterMapping', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_TSECOMMAND_ARGUMENTMAPPINGENTRY, _TSECOMMAND_PARAMETERMAPPINGENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=81,
  serialized_end=442,
)

_TSECOMMAND_ARGUMENTMAPPINGENTRY.fields_by_name['value'].message_type = yamcs_dot_protobuf_dot_yamcs__pb2._VALUE
_TSECOMMAND_ARGUMENTMAPPINGENTRY.containing_type = _TSECOMMAND
_TSECOMMAND_PARAMETERMAPPINGENTRY.containing_type = _TSECOMMAND
_TSECOMMAND.fields_by_name['argumentMapping'].message_type = _TSECOMMAND_ARGUMENTMAPPINGENTRY
_TSECOMMAND.fields_by_name['parameterMapping'].message_type = _TSECOMMAND_PARAMETERMAPPINGENTRY
DESCRIPTOR.message_types_by_name['TseCommand'] = _TSECOMMAND
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TseCommand = _reflection.GeneratedProtocolMessageType('TseCommand', (_message.Message,), dict(

  ArgumentMappingEntry = _reflection.GeneratedProtocolMessageType('ArgumentMappingEntry', (_message.Message,), dict(
    DESCRIPTOR = _TSECOMMAND_ARGUMENTMAPPINGENTRY,
    __module__ = 'yamcs.protobuf.tse.tse_pb2'
    # @@protoc_insertion_point(class_scope:yamcs.protobuf.tse.TseCommand.ArgumentMappingEntry)
    ))
  ,

  ParameterMappingEntry = _reflection.GeneratedProtocolMessageType('ParameterMappingEntry', (_message.Message,), dict(
    DESCRIPTOR = _TSECOMMAND_PARAMETERMAPPINGENTRY,
    __module__ = 'yamcs.protobuf.tse.tse_pb2'
    # @@protoc_insertion_point(class_scope:yamcs.protobuf.tse.TseCommand.ParameterMappingEntry)
    ))
  ,
  DESCRIPTOR = _TSECOMMAND,
  __module__ = 'yamcs.protobuf.tse.tse_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.tse.TseCommand)
  ))
_sym_db.RegisterMessage(TseCommand)
_sym_db.RegisterMessage(TseCommand.ArgumentMappingEntry)
_sym_db.RegisterMessage(TseCommand.ParameterMappingEntry)


DESCRIPTOR._options = None
_TSECOMMAND_ARGUMENTMAPPINGENTRY._options = None
_TSECOMMAND_PARAMETERMAPPINGENTRY._options = None
# @@protoc_insertion_point(module_scope)



from yamcs.core.client import BaseClient
from yamcs.protobuf.rest import rest_pb2
from yamcs.storage.model import Bucket, ObjectListing


class Client(object):
    """
    Client for working with buckets and objects managed by Yamcs.
    """

    def __init__(self, address, **kwargs):
        """
        :param str address: The address of Yamcs in the format 'hostname:port'
        :param bool ssl: Whether SSL encryption is expected
        :param .Credentials credentials: Credentials for when the server is secured
        :param str user_agent: Optionally override the default user agent
        """
        super(Client, self).__init__()
        self._client = BaseClient(address, **kwargs)

    def list_buckets(self, instance):
        """
        List the buckets for an instance.

        :param str instance: A Yamcs instance name.
        :rtype: ~collections.Iterable[.Bucket]
        """
        # Server does not do pagination on listings of this resource.
        # Return an iterator anyway for similarity with other API methods
        response = self._client.get_proto(path='/buckets/' + instance)
        message = rest_pb2.ListBucketsResponse()
        message.ParseFromString(response.content)
        buckets = getattr(message, 'bucket')
        return iter([
            Bucket(bucket, instance, self) for bucket in buckets])

    def list_objects(self, instance, bucket_name, prefix=None, delimiter=None):
        """
        List the objects for a bucket.

        :param str instance: A Yamcs instance name.
        :param str bucket_name: The name of the bucket.
        :param str prefix: If specified, only objects that start with this
                           prefix are listed.
        :param str delimiter: If specified, return only objects whose name
                              do not contain the delimiter after the prefix.
                              For the other objects, the response contains
                              (in the prefix response parameter) the name
                              truncated after the delimiter. Duplicates are
                              omitted.
        """
        url = '/buckets/{}/{}'.format(instance, bucket_name)
        params = {}
        if prefix is not None:
            params['prefix'] = prefix
        if delimiter is not None:
            params['delimiter'] = delimiter
        response = self._client.get_proto(path=url, params=params)
        message = rest_pb2.ListObjectsResponse()
        message.ParseFromString(response.content)
        return ObjectListing(message, instance, bucket_name, self)

    def create_bucket(self, instance, bucket_name):
        """
        Create a new bucket in the specified instance.

        :param str instance: A Yamcs instance name.
        :param str bucket_name: The name of the bucket.
        """
        req = rest_pb2.CreateBucketRequest()
        req.name = bucket_name
        url = '/buckets/{}'.format(instance)
        self._client.post_proto(url, data=req.SerializeToString())

    def remove_bucket(self, instance, bucket_name):
        """
        Remove a bucket from the specified instance.

        :param str instance: A Yamcs instance name.
        :param str bucket_name: The name of the bucket.
        """
        url = '/buckets/{}/{}'.format(instance, bucket_name)
        self._client.delete_proto(url)

    def download_object(self, instance, bucket_name, object_name):
        """
        Download an object.

        :param str instance: A Yamcs instance name.
        :param str bucket_name: The name of the bucket.
        :param str object_name: The object to fetch.
        """
        url = '/buckets/{}/{}/{}'.format(instance, bucket_name, object_name)
        response = self._client.get_proto(path=url)
        return response.content

    def upload_object(self, instance, bucket_name, object_name, file_obj,
                      content_type=None):
        """
        Upload an object to a bucket.

        :param str instance: A Yamcs instance name.
        :param str bucket_name: The name of the bucket.
        :param str object_name: The target name of the object.
        :param file file_obj: The file (or file-like object) to upload.
        :param str content_type: The content type associated to this object.
                                 This is mainly useful when accessing an object
                                 directly via a web browser. If unspecified, a
                                 content type *may* be automatically derived
                                 from the specified ``file_obj``.
        """
        url = '/buckets/{}/{}/{}'.format(instance, bucket_name, object_name)
        with open(file_obj, 'rb') as f:
            if content_type:
                files = {object_name: (object_name, f, content_type)}
            else:
                files = {object_name: (object_name, f)}
            self._client.request(path=url, method='post', files=files)

    def remove_object(self, instance, bucket_name, object_name):
        """
        Remove an object from a bucket.

        :param str instance: A Yamcs instance name.
        :param str bucket_name: The name of the bucket.
        :param str object_name: The object to remove.
        """
        url = '/buckets/{}/{}/{}'.format(instance, bucket_name, object_name)
        self._client.delete_proto(url)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: yamcs/protobuf/archive/archive.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from yamcs.protobuf import yamcs_pb2 as yamcs_dot_protobuf_dot_yamcs__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='yamcs/protobuf/archive/archive.proto',
  package='yamcs.protobuf.archive',
  syntax='proto2',
  serialized_options=_b('\n\022org.yamcs.protobuf'),
  serialized_pb=_b('\n$yamcs/protobuf/archive/archive.proto\x12\x16yamcs.protobuf.archive\x1a\x1ayamcs/protobuf/yamcs.proto\"@\n\nColumnData\x12\x0c\n\x04name\x18\x01 \x01(\t\x12$\n\x05value\x18\x02 \x01(\x0b\x32\x15.yamcs.protobuf.Value\"P\n\nStreamData\x12\x0e\n\x06stream\x18\x01 \x01(\t\x12\x32\n\x06\x63olumn\x18\x02 \x03(\x0b\x32\".yamcs.protobuf.archive.ColumnData\"\x8d\x01\n\tTableData\x12=\n\x06record\x18\x01 \x03(\x0b\x32-.yamcs.protobuf.archive.TableData.TableRecord\x1a\x41\n\x0bTableRecord\x12\x32\n\x06\x63olumn\x18\x01 \x03(\x0b\x32\".yamcs.protobuf.archive.ColumnData\"^\n\nColumnInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\x12\x34\n\tenumValue\x18\x03 \x03(\x0b\x32!.yamcs.protobuf.archive.EnumValue\")\n\tEnumValue\x12\r\n\x05value\x18\x01 \x01(\x05\x12\r\n\x05label\x18\x02 \x01(\t\"\xcc\x02\n\tTableInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x35\n\tkeyColumn\x18\x02 \x03(\x0b\x32\".yamcs.protobuf.archive.ColumnInfo\x12\x37\n\x0bvalueColumn\x18\x03 \x03(\x0b\x32\".yamcs.protobuf.archive.ColumnInfo\x12\x0e\n\x06script\x18\x04 \x01(\t\x12\x17\n\x0fhistogramColumn\x18\x05 \x03(\t\x12\x15\n\rstorageEngine\x18\x06 \x01(\t\x12\x15\n\rformatVersion\x18\x07 \x01(\x05\x12\x12\n\ntablespace\x18\x08 \x01(\t\x12\x12\n\ncompressed\x18\t \x01(\x08\x12\x42\n\x10partitioningInfo\x18\n \x01(\x0b\x32(.yamcs.protobuf.archive.PartitioningInfo\"\xf7\x01\n\x10PartitioningInfo\x12G\n\x04type\x18\x01 \x01(\x0e\x32\x39.yamcs.protobuf.archive.PartitioningInfo.PartitioningType\x12\x12\n\ntimeColumn\x18\x02 \x01(\t\x12\x1b\n\x13timePartitionSchema\x18\x03 \x01(\t\x12\x13\n\x0bvalueColumn\x18\x04 \x01(\t\x12\x17\n\x0fvalueColumnType\x18\x05 \x01(\t\";\n\x10PartitioningType\x12\x08\n\x04TIME\x10\x01\x12\t\n\x05VALUE\x10\x02\x12\x12\n\x0eTIME_AND_VALUE\x10\x03\"q\n\nStreamInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x32\n\x06\x63olumn\x18\x02 \x03(\x0b\x32\".yamcs.protobuf.archive.ColumnInfo\x12\x0e\n\x06script\x18\x03 \x01(\t\x12\x11\n\tdataCount\x18\x04 \x01(\x03\"!\n\x0f\x45ventSourceInfo\x12\x0e\n\x06source\x18\x01 \x03(\t\"#\n\x12ParameterGroupInfo\x12\r\n\x05group\x18\x01 \x03(\t\"]\n\rIndexResponse\x12\x31\n\x05group\x18\x01 \x03(\x0b\x32\".yamcs.protobuf.archive.IndexGroup\x12\x19\n\x11\x63ontinuationToken\x18\x02 \x01(\t\"j\n\nIndexGroup\x12)\n\x02id\x18\x01 \x01(\x0b\x32\x1d.yamcs.protobuf.NamedObjectId\x12\x31\n\x05\x65ntry\x18\x02 \x03(\x0b\x32\".yamcs.protobuf.archive.IndexEntry\"[\n\nIndexEntry\x12\r\n\x05start\x18\x01 \x01(\t\x12\x0c\n\x04stop\x18\x02 \x01(\t\x12\r\n\x05\x63ount\x18\x03 \x01(\x05\x12\x10\n\x08seqStart\x18\x04 \x01(\x03\x12\x0f\n\x07seqStop\x18\x05 \x01(\x03\"&\n\x16GetPacketNamesResponse\x12\x0c\n\x04name\x18\x01 \x03(\t\"&\n\x11\x45xecuteSqlRequest\x12\x11\n\tstatement\x18\x01 \x01(\t\"$\n\x12\x45xecuteSqlResponse\x12\x0e\n\x06result\x18\x01 \x01(\tB\x14\n\x12org.yamcs.protobuf')
  ,
  dependencies=[yamcs_dot_protobuf_dot_yamcs__pb2.DESCRIPTOR,])



_PARTITIONINGINFO_PARTITIONINGTYPE = _descriptor.EnumDescriptor(
  name='PartitioningType',
  full_name='yamcs.protobuf.archive.PartitioningInfo.PartitioningType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TIME', index=0, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VALUE', index=1, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TIME_AND_VALUE', index=2, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1047,
  serialized_end=1106,
)
_sym_db.RegisterEnumDescriptor(_PARTITIONINGINFO_PARTITIONINGTYPE)


_COLUMNDATA = _descriptor.Descriptor(
  name='ColumnData',
  full_name='yamcs.protobuf.archive.ColumnData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.archive.ColumnData.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='yamcs.protobuf.archive.ColumnData.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=92,
  serialized_end=156,
)


_STREAMDATA = _descriptor.Descriptor(
  name='StreamData',
  full_name='yamcs.protobuf.archive.StreamData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stream', full_name='yamcs.protobuf.archive.StreamData.stream', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='column', full_name='yamcs.protobuf.archive.StreamData.column', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=158,
  serialized_end=238,
)


_TABLEDATA_TABLERECORD = _descriptor.Descriptor(
  name='TableRecord',
  full_name='yamcs.protobuf.archive.TableData.TableRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='column', full_name='yamcs.protobuf.archive.TableData.TableRecord.column', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=317,
  serialized_end=382,
)

_TABLEDATA = _descriptor.Descriptor(
  name='TableData',
  full_name='yamcs.protobuf.archive.TableData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='record', full_name='yamcs.protobuf.archive.TableData.record', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_TABLEDATA_TABLERECORD, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=241,
  serialized_end=382,
)


_COLUMNINFO = _descriptor.Descriptor(
  name='ColumnInfo',
  full_name='yamcs.protobuf.archive.ColumnInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.archive.ColumnInfo.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='yamcs.protobuf.archive.ColumnInfo.type', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='enumValue', full_name='yamcs.protobuf.archive.ColumnInfo.enumValue', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=384,
  serialized_end=478,
)


_ENUMVALUE = _descriptor.Descriptor(
  name='EnumValue',
  full_name='yamcs.protobuf.archive.EnumValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='yamcs.protobuf.archive.EnumValue.value', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='label', full_name='yamcs.protobuf.archive.EnumValue.label', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=480,
  serialized_end=521,
)


_TABLEINFO = _descriptor.Descriptor(
  name='TableInfo',
  full_name='yamcs.protobuf.archive.TableInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.archive.TableInfo.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='keyColumn', full_name='yamcs.protobuf.archive.TableInfo.keyColumn', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='valueColumn', full_name='yamcs.protobuf.archive.TableInfo.valueColumn', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='script', full_name='yamcs.protobuf.archive.TableInfo.script', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='histogramColumn', full_name='yamcs.protobuf.archive.TableInfo.histogramColumn', index=4,
      number=5, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='storageEngine', full_name='yamcs.protobuf.archive.TableInfo.storageEngine', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='formatVersion', full_name='yamcs.protobuf.archive.TableInfo.formatVersion', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tablespace', full_name='yamcs.protobuf.archive.TableInfo.tablespace', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='compressed', full_name='yamcs.protobuf.archive.TableInfo.compressed', index=8,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='partitioningInfo', full_name='yamcs.protobuf.archive.TableInfo.partitioningInfo', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=524,
  serialized_end=856,
)


_PARTITIONINGINFO = _descriptor.Descriptor(
  name='PartitioningInfo',
  full_name='yamcs.protobuf.archive.PartitioningInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='yamcs.protobuf.archive.PartitioningInfo.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timeColumn', full_name='yamcs.protobuf.archive.PartitioningInfo.timeColumn', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timePartitionSchema', full_name='yamcs.protobuf.archive.PartitioningInfo.timePartitionSchema', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='valueColumn', full_name='yamcs.protobuf.archive.PartitioningInfo.valueColumn', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='valueColumnType', full_name='yamcs.protobuf.archive.PartitioningInfo.valueColumnType', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _PARTITIONINGINFO_PARTITIONINGTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=859,
  serialized_end=1106,
)


_STREAMINFO = _descriptor.Descriptor(
  name='StreamInfo',
  full_name='yamcs.protobuf.archive.StreamInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.archive.StreamInfo.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='column', full_name='yamcs.protobuf.archive.StreamInfo.column', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='script', full_name='yamcs.protobuf.archive.StreamInfo.script', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataCount', full_name='yamcs.protobuf.archive.StreamInfo.dataCount', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1108,
  serialized_end=1221,
)


_EVENTSOURCEINFO = _descriptor.Descriptor(
  name='EventSourceInfo',
  full_name='yamcs.protobuf.archive.EventSourceInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='source', full_name='yamcs.protobuf.archive.EventSourceInfo.source', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1223,
  serialized_end=1256,
)


_PARAMETERGROUPINFO = _descriptor.Descriptor(
  name='ParameterGroupInfo',
  full_name='yamcs.protobuf.archive.ParameterGroupInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='group', full_name='yamcs.protobuf.archive.ParameterGroupInfo.group', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1258,
  serialized_end=1293,
)


_INDEXRESPONSE = _descriptor.Descriptor(
  name='IndexResponse',
  full_name='yamcs.protobuf.archive.IndexResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='group', full_name='yamcs.protobuf.archive.IndexResponse.group', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='continuationToken', full_name='yamcs.protobuf.archive.IndexResponse.continuationToken', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1295,
  serialized_end=1388,
)


_INDEXGROUP = _descriptor.Descriptor(
  name='IndexGroup',
  full_name='yamcs.protobuf.archive.IndexGroup',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='yamcs.protobuf.archive.IndexGroup.id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='entry', full_name='yamcs.protobuf.archive.IndexGroup.entry', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1390,
  serialized_end=1496,
)


_INDEXENTRY = _descriptor.Descriptor(
  name='IndexEntry',
  full_name='yamcs.protobuf.archive.IndexEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='start', full_name='yamcs.protobuf.archive.IndexEntry.start', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stop', full_name='yamcs.protobuf.archive.IndexEntry.stop', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='count', full_name='yamcs.protobuf.archive.IndexEntry.count', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seqStart', full_name='yamcs.protobuf.archive.IndexEntry.seqStart', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seqStop', full_name='yamcs.protobuf.archive.IndexEntry.seqStop', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1498,
  serialized_end=1589,
)


_GETPACKETNAMESRESPONSE = _descriptor.Descriptor(
  name='GetPacketNamesResponse',
  full_name='yamcs.protobuf.archive.GetPacketNamesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.archive.GetPacketNamesResponse.name', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1591,
  serialized_end=1629,
)


_EXECUTESQLREQUEST = _descriptor.Descriptor(
  name='ExecuteSqlRequest',
  full_name='yamcs.protobuf.archive.ExecuteSqlRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='statement', full_name='yamcs.protobuf.archive.ExecuteSqlRequest.statement', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1631,
  serialized_end=1669,
)


_EXECUTESQLRESPONSE = _descriptor.Descriptor(
  name='ExecuteSqlResponse',
  full_name='yamcs.protobuf.archive.ExecuteSqlResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='yamcs.protobuf.archive.ExecuteSqlResponse.result', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1671,
  serialized_end=1707,
)

_COLUMNDATA.fields_by_name['value'].message_type = yamcs_dot_protobuf_dot_yamcs__pb2._VALUE
_STREAMDATA.fields_by_name['column'].message_type = _COLUMNDATA
_TABLEDATA_TABLERECORD.fields_by_name['column'].message_type = _COLUMNDATA
_TABLEDATA_TABLERECORD.containing_type = _TABLEDATA
_TABLEDATA.fields_by_name['record'].message_type = _TABLEDATA_TABLERECORD
_COLUMNINFO.fields_by_name['enumValue'].message_type = _ENUMVALUE
_TABLEINFO.fields_by_name['keyColumn'].message_type = _COLUMNINFO
_TABLEINFO.fields_by_name['valueColumn'].message_type = _COLUMNINFO
_TABLEINFO.fields_by_name['partitioningInfo'].message_type = _PARTITIONINGINFO
_PARTITIONINGINFO.fields_by_name['type'].enum_type = _PARTITIONINGINFO_PARTITIONINGTYPE
_PARTITIONINGINFO_PARTITIONINGTYPE.containing_type = _PARTITIONINGINFO
_STREAMINFO.fields_by_name['column'].message_type = _COLUMNINFO
_INDEXRESPONSE.fields_by_name['group'].message_type = _INDEXGROUP
_INDEXGROUP.fields_by_name['id'].message_type = yamcs_dot_protobuf_dot_yamcs__pb2._NAMEDOBJECTID
_INDEXGROUP.fields_by_name['entry'].message_type = _INDEXENTRY
DESCRIPTOR.message_types_by_name['ColumnData'] = _COLUMNDATA
DESCRIPTOR.message_types_by_name['StreamData'] = _STREAMDATA
DESCRIPTOR.message_types_by_name['TableData'] = _TABLEDATA
DESCRIPTOR.message_types_by_name['ColumnInfo'] = _COLUMNINFO
DESCRIPTOR.message_types_by_name['EnumValue'] = _ENUMVALUE
DESCRIPTOR.message_types_by_name['TableInfo'] = _TABLEINFO
DESCRIPTOR.message_types_by_name['PartitioningInfo'] = _PARTITIONINGINFO
DESCRIPTOR.message_types_by_name['StreamInfo'] = _STREAMINFO
DESCRIPTOR.message_types_by_name['EventSourceInfo'] = _EVENTSOURCEINFO
DESCRIPTOR.message_types_by_name['ParameterGroupInfo'] = _PARAMETERGROUPINFO
DESCRIPTOR.message_types_by_name['IndexResponse'] = _INDEXRESPONSE
DESCRIPTOR.message_types_by_name['IndexGroup'] = _INDEXGROUP
DESCRIPTOR.message_types_by_name['IndexEntry'] = _INDEXENTRY
DESCRIPTOR.message_types_by_name['GetPacketNamesResponse'] = _GETPACKETNAMESRESPONSE
DESCRIPTOR.message_types_by_name['ExecuteSqlRequest'] = _EXECUTESQLREQUEST
DESCRIPTOR.message_types_by_name['ExecuteSqlResponse'] = _EXECUTESQLRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ColumnData = _reflection.GeneratedProtocolMessageType('ColumnData', (_message.Message,), dict(
  DESCRIPTOR = _COLUMNDATA,
  __module__ = 'yamcs.protobuf.archive.archive_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.ColumnData)
  ))
_sym_db.RegisterMessage(ColumnData)

StreamData = _reflection.GeneratedProtocolMessageType('StreamData', (_message.Message,), dict(
  DESCRIPTOR = _STREAMDATA,
  __module__ = 'yamcs.protobuf.archive.archive_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.StreamData)
  ))
_sym_db.RegisterMessage(StreamData)

TableData = _reflection.GeneratedProtocolMessageType('TableData', (_message.Message,), dict(

  TableRecord = _reflection.GeneratedProtocolMessageType('TableRecord', (_message.Message,), dict(
    DESCRIPTOR = _TABLEDATA_TABLERECORD,
    __module__ = 'yamcs.protobuf.archive.archive_pb2'
    # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.TableData.TableRecord)
    ))
  ,
  DESCRIPTOR = _TABLEDATA,
  __module__ = 'yamcs.protobuf.archive.archive_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.TableData)
  ))
_sym_db.RegisterMessage(TableData)
_sym_db.RegisterMessage(TableData.TableRecord)

ColumnInfo = _reflection.GeneratedProtocolMessageType('ColumnInfo', (_message.Message,), dict(
  DESCRIPTOR = _COLUMNINFO,
  __module__ = 'yamcs.protobuf.archive.archive_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.ColumnInfo)
  ))
_sym_db.RegisterMessage(ColumnInfo)

EnumValue = _reflection.GeneratedProtocolMessageType('EnumValue', (_message.Message,), dict(
  DESCRIPTOR = _ENUMVALUE,
  __module__ = 'yamcs.protobuf.archive.archive_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.EnumValue)
  ))
_sym_db.RegisterMessage(EnumValue)

TableInfo = _reflection.GeneratedProtocolMessageType('TableInfo', (_message.Message,), dict(
  DESCRIPTOR = _TABLEINFO,
  __module__ = 'yamcs.protobuf.archive.archive_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.TableInfo)
  ))
_sym_db.RegisterMessage(TableInfo)

PartitioningInfo = _reflection.GeneratedProtocolMessageType('PartitioningInfo', (_message.Message,), dict(
  DESCRIPTOR = _PARTITIONINGINFO,
  __module__ = 'yamcs.protobuf.archive.archive_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.PartitioningInfo)
  ))
_sym_db.RegisterMessage(PartitioningInfo)

StreamInfo = _reflection.GeneratedProtocolMessageType('StreamInfo', (_message.Message,), dict(
  DESCRIPTOR = _STREAMINFO,
  __module__ = 'yamcs.protobuf.archive.archive_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.StreamInfo)
  ))
_sym_db.RegisterMessage(StreamInfo)

EventSourceInfo = _reflection.GeneratedProtocolMessageType('EventSourceInfo', (_message.Message,), dict(
  DESCRIPTOR = _EVENTSOURCEINFO,
  __module__ = 'yamcs.protobuf.archive.archive_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.EventSourceInfo)
  ))
_sym_db.RegisterMessage(EventSourceInfo)

ParameterGroupInfo = _reflection.GeneratedProtocolMessageType('ParameterGroupInfo', (_message.Message,), dict(
  DESCRIPTOR = _PARAMETERGROUPINFO,
  __module__ = 'yamcs.protobuf.archive.archive_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.ParameterGroupInfo)
  ))
_sym_db.RegisterMessage(ParameterGroupInfo)

IndexResponse = _reflection.GeneratedProtocolMessageType('IndexResponse', (_message.Message,), dict(
  DESCRIPTOR = _INDEXRESPONSE,
  __module__ = 'yamcs.protobuf.archive.archive_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.IndexResponse)
  ))
_sym_db.RegisterMessage(IndexResponse)

IndexGroup = _reflection.GeneratedProtocolMessageType('IndexGroup', (_message.Message,), dict(
  DESCRIPTOR = _INDEXGROUP,
  __module__ = 'yamcs.protobuf.archive.archive_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.IndexGroup)
  ))
_sym_db.RegisterMessage(IndexGroup)

IndexEntry = _reflection.GeneratedProtocolMessageType('IndexEntry', (_message.Message,), dict(
  DESCRIPTOR = _INDEXENTRY,
  __module__ = 'yamcs.protobuf.archive.archive_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.IndexEntry)
  ))
_sym_db.RegisterMessage(IndexEntry)

GetPacketNamesResponse = _reflection.GeneratedProtocolMessageType('GetPacketNamesResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETPACKETNAMESRESPONSE,
  __module__ = 'yamcs.protobuf.archive.archive_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.GetPacketNamesResponse)
  ))
_sym_db.RegisterMessage(GetPacketNamesResponse)

ExecuteSqlRequest = _reflection.GeneratedProtocolMessageType('ExecuteSqlRequest', (_message.Message,), dict(
  DESCRIPTOR = _EXECUTESQLREQUEST,
  __module__ = 'yamcs.protobuf.archive.archive_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.ExecuteSqlRequest)
  ))
_sym_db.RegisterMessage(ExecuteSqlRequest)

ExecuteSqlResponse = _reflection.GeneratedProtocolMessageType('ExecuteSqlResponse', (_message.Message,), dict(
  DESCRIPTOR = _EXECUTESQLRESPONSE,
  __module__ = 'yamcs.protobuf.archive.archive_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.archive.ExecuteSqlResponse)
  ))
_sym_db.RegisterMessage(ExecuteSqlResponse)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



from __future__ import print_function

import logging
from collections import OrderedDict
from datetime import datetime, timedelta


def to_isostring(dt):
    """
    Converts the given datetime to an ISO String.
    This assumes the datetime is UTC.
    """
    if dt.tzinfo is not None and dt.tzinfo.utcoffset(dt) > timedelta(0):
        logging.warn('Warning: aware datetimes are interpreted as if they were naive')

    # -3 to change microseconds to milliseconds
    return dt.strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + 'Z'


def parse_isostring(isostring):
    """
    Parse the ISO String to a native ``datetime``.
    """
    if not isostring:
        return None
    return datetime.strptime(isostring.replace('Z', 'GMT'),
                             '%Y-%m-%dT%H:%M:%S.%f%Z')


def parse_value(proto):
    """
    Convers a Protobuf `Value` from the API into a python native value
    """
    if proto.HasField('floatValue'):
        return proto.floatValue
    elif proto.HasField('doubleValue'):
        return proto.doubleValue
    elif proto.HasField('sint32Value'):
        return proto.sint32Value
    elif proto.HasField('uint32Value'):
        return proto.uint32Value
    elif proto.HasField('binaryValue'):
        return proto.binaryValue
    elif proto.HasField('timestampValue'):
        # Don't use the actual 'timestampValue' field, it contains a number
        # that is difficult to interpret on the client. Instead parse from
        # the ISO String also set by Yamcs.
        return parse_isostring(proto.stringValue)
    elif proto.HasField('stringValue'):
        return proto.stringValue
    elif proto.HasField('uint64Value'):
        return proto.uint64Value
    elif proto.HasField('sint64Value'):
        return proto.sint64Value
    elif proto.HasField('booleanValue'):
        return proto.booleanValue
    elif proto.HasField('arrayValue'):
        return [parse_value(v) for v in proto.arrayValue]
    elif proto.HasField('aggregateValue'):
        return OrderedDict(zip(proto.aggregateValue.name, proto.aggregateValue.value))
    else:
        logging.warn('Unrecognized value type for update %s', proto)
        return None


def adapt_name_for_rest(name):
    """
    Modifies a user-provided name for use in API calls.
    Basically we want an alias like 'MDB:OPS Name/SIMULATOR_BatteryVoltage2'
    to be prepended with a slash.
    """
    if name.startswith('/'):
        return name
    elif not '/' in name:
        raise ValueError('Provided name is not a fully-qualified XTCE name.')
    return '/' + name



from yamcs.core.helpers import parse_isostring


class Bucket(object):

    def __init__(self, proto, instance, client):
        self._proto = proto
        self._instance = instance
        self._client = client

    @property
    def name(self):
        """Name of this bucket."""
        return self._proto.name

    @property
    def object_count(self):
        """Number of objects in this bucket."""
        return self._proto.numObjects

    @property
    def size(self):
        """Total size in bytes of this bucket (excluding metadata)."""
        return self._proto.size

    def list_objects(self, prefix=None, delimiter=None):
        """
        List the objects for this bucket.

        :param str prefix: If specified, only objects that start with this
                           prefix are listed.
        :param str delimiter: If specified, return only objects whose name
                              do not contain the delimiter after the prefix.
                              For the other objects, the response contains
                              (in the prefix response parameter) the name
                              truncated after the delimiter. Duplicates are
                              omitted.
        """
        return self._client.list_objects(
            instance=self._instance, bucket_name=self.name, prefix=prefix,
            delimiter=delimiter)

    def download_object(self, object_name):
        """
        Download an object.

        :param str object_name: The object to fetch.
        """
        return self._client.download_object(
            self._instance, self.name, object_name)

    def upload_object(self, object_name, file_obj):
        """
        Upload an object to this bucket.

        :param str object_name: The target name of the object.
        :param file file_obj: The file (or file-like object) to upload.
        :param str content_type: The content type associated to this object.
                                 This is mainly useful when accessing an object
                                 directly via a web browser. If unspecified, a
                                 content type *may* be automatically derived
                                 from the specified ``file_obj``.
        """
        return self._client.upload_object(
            self._instance, self.name, object_name, file_obj)

    def delete_object(self, object_name):
        """
        Remove an object from this bucket.

        :param str object_name: The object to remove.
        """
        self._client.remove_object(self._instance, self.name, object_name)

    def delete(self):
        """
        Remove this bucket in its entirety.
        """
        self._client.remove_bucket(self._instance, self.name)

    def __str__(self):
        return self.name


class ObjectListing(object):

    def __init__(self, proto, instance, bucket, client):
        self._proto = proto
        self._instance = instance
        self._bucket = bucket
        self._client = client

    @property
    def prefixes(self):
        """
        The prefixes in this listing.

        :type: str[]
        """
        return [p for p in self._proto.prefix]

    @property
    def objects(self):
        """
        The objects in this listing.

        :type: List[:class:`.ObjectInfo`]
        """
        return [ObjectInfo(o, self._instance, self._bucket, self._client)
                for o in self._proto.object]


class ObjectInfo(object):

    def __init__(self, proto, instance, bucket, client):
        self._proto = proto
        self._instance = instance
        self._bucket = bucket
        self._client = client

    @property
    def name(self):
        """The name of this object."""
        return self._proto.name

    @property
    def size(self):
        """Size in bytes of this object (excluding metadata)."""
        return self._proto.size

    @property
    def created(self):
        """
        Return when this object was created (or re-created).

        :type: :class:`~datetime.datetime`
        """
        if self._proto.HasField('created'):
            return parse_isostring(self._proto.created)
        return None

    def delete(self):
        """Remove this object."""
        self._client.remove_object(self._instance, self._bucket, self.name)

    def download(self):
        """Download this object."""
        return self._client.download_object(
            self._instance, self._bucket, self.name)

    def upload(self, file_obj):
        """
        Replace the content of this object.

        :param file file_obj: The file (or file-like object) to upload.
        """
        return self._client.upload_object(
            self._instance, self._bucket, self.name, file_obj)

    def __str__(self):
        return '{} ({} bytes, created {})'.format(
            self.name, self.size, self.created)
